# OpenAI Python bindings.
#
# Originally forked from the MIT-licensed Stripe Python bindings.

import os
from contextvars import ContextVar
from typing import Optional, TYPE_CHECKING

from openai.api_resources import (
    Audio,
    ChatCompletion,
    Completion,
    Customer,
    Edit,
    Deployment,
    Embedding,
    Engine,
    ErrorObject,
    File,
    FineTune,
    Image,
    Model,
    Moderation,
)
from openai.error import APIError, InvalidRequestError, OpenAIError

if TYPE_CHECKING:
    from aiohttp import ClientSession

api_key = os.environ.get("OPENAI_API_KEY")
# Path of a file with an API key, whose contents can change. Supercedes
# `api_key` if set.  The main use case is volume-mounted Kubernetes secrets,
# which are updated automatically.
api_key_path: Optional[str] = os.environ.get("OPENAI_API_KEY_PATH")

organization = os.environ.get("OPENAI_ORGANIZATION")
api_base = os.environ.get("OPENAI_API_BASE", "https://api.openai.com/v1")
api_type = os.environ.get("OPENAI_API_TYPE", "open_ai")
api_version = (
    "2023-03-15-preview" if api_type in ("azure", "azure_ad", "azuread") else None
)
verify_ssl_certs = True  # No effect. Certificates are always verified.
proxy = None
app_info = None
enable_telemetry = False  # Ignored; the telemetry feature was removed.
ca_bundle_path = None  # No longer used, feature was removed
debug = False
log = None  # Set to either 'debug' or 'info', controls console logging

aiosession: ContextVar[Optional["ClientSession"]] = ContextVar(
    "aiohttp-session", default=None
)  # Acts as a global aiohttp ClientSession that reuses connections.
# This is user-supplied; otherwise, a session is remade for each request.

__all__ = [
    "APIError",
    "Audio",
    "ChatCompletion",
    "Completion",
    "Customer",
    "Edit",
    "Image",
    "Deployment",
    "Embedding",
    "Engine",
    "ErrorObject",
    "File",
    "FineTune",
    "InvalidRequestError",
    "Model",
    "Moderation",
    "OpenAIError",
    "api_base",
    "api_key",
    "api_type",
    "api_key_path",
    "api_version",
    "app_info",
    "ca_bundle_path",
    "debug",
    "enable_telemetry",
    "log",
    "organization",
    "proxy",
    "verify_ssl_certs",
]



from typing import Any, List

import openai
from openai import api_requestor, util
from openai.api_resources.abstract import APIResource


class Audio(APIResource):
    OBJECT_NAME = "audio"

    @classmethod
    def _get_url(cls, action):
        return cls.class_url() + f"/{action}"

    @classmethod
    def _prepare_request(
        cls,
        file,
        filename,
        model,
        api_key=None,
        api_base=None,
        api_type=None,
        api_version=None,
        organization=None,
        **params,
    ):
        requestor = api_requestor.APIRequestor(
            api_key,
            api_base=api_base or openai.api_base,
            api_type=api_type,
            api_version=api_version,
            organization=organization,
        )
        files: List[Any] = []
        data = {
            "model": model,
            **params,
        }
        files.append(("file", (filename, file, "application/octet-stream")))
        return requestor, files, data

    @classmethod
    def transcribe(
        cls,
        model,
        file,
        api_key=None,
        api_base=None,
        api_type=None,
        api_version=None,
        organization=None,
        **params,
    ):
        requestor, files, data = cls._prepare_request(
            file=file,
            filename=file.name,
            model=model,
            api_key=api_key,
            api_base=api_base,
            api_type=api_type,
            **params,
        )
        url = cls._get_url("transcriptions")
        response, _, api_key = requestor.request("post", url, files=files, params=data)
        return util.convert_to_openai_object(
            response, api_key, api_version, organization
        )

    @classmethod
    def translate(
        cls,
        model,
        file,
        api_key=None,
        api_base=None,
        api_type=None,
        api_version=None,
        organization=None,
        **params,
    ):
        requestor, files, data = cls._prepare_request(
            file=file,
            filename=file.name,
            model=model,
            api_key=api_key,
            api_base=api_base,
            api_type=api_type,
            **params,
        )
        url = cls._get_url("translations")
        response, _, api_key = requestor.request("post", url, files=files, params=data)
        return util.convert_to_openai_object(
            response, api_key, api_version, organization
        )

    @classmethod
    def transcribe_raw(
        cls,
        model,
        file,
        filename,
        api_key=None,
        api_base=None,
        api_type=None,
        api_version=None,
        organization=None,
        **params,
    ):
        requestor, files, data = cls._prepare_request(
            file=file,
            filename=filename,
            model=model,
            api_key=api_key,
            api_base=api_base,
            api_type=api_type,
            **params,
        )
        url = cls._get_url("transcriptions")
        response, _, api_key = requestor.request("post", url, files=files, params=data)
        return util.convert_to_openai_object(
            response, api_key, api_version, organization
        )

    @classmethod
    def translate_raw(
        cls,
        model,
        file,
        filename,
        api_key=None,
        api_base=None,
        api_type=None,
        api_version=None,
        organization=None,
        **params,
    ):
        requestor, files, data = cls._prepare_request(
            file=file,
            filename=filename,
            model=model,
            api_key=api_key,
            api_base=api_base,
            api_type=api_type,
            **params,
        )
        url = cls._get_url("translations")
        response, _, api_key = requestor.request("post", url, files=files, params=data)
        return util.convert_to_openai_object(
            response, api_key, api_version, organization
        )

    @classmethod
    async def atranscribe(
        cls,
        model,
        file,
        api_key=None,
        api_base=None,
        api_type=None,
        api_version=None,
        organization=None,
        **params,
    ):
        requestor, files, data = cls._prepare_request(
            file=file,
            filename=file.name,
            model=model,
            api_key=api_key,
            api_base=api_base,
            api_type=api_type,
            **params,
        )
        url = cls._get_url("transcriptions")
        response, _, api_key = await requestor.arequest(
            "post", url, files=files, params=data
        )
        return util.convert_to_openai_object(
            response, api_key, api_version, organization
        )

    @classmethod
    async def atranslate(
        cls,
        model,
        file,
        api_key=None,
        api_base=None,
        api_type=None,
        api_version=None,
        organization=None,
        **params,
    ):
        requestor, files, data = cls._prepare_request(
            file=file,
            filename=file.name,
            model=model,
            api_key=api_key,
            api_base=api_base,
            api_type=api_type,
            **params,
        )
        url = cls._get_url("translations")
        response, _, api_key = await requestor.arequest(
            "post", url, files=files, params=data
        )
        return util.convert_to_openai_object(
            response, api_key, api_version, organization
        )

    @classmethod
    async def atranscribe_raw(
        cls,
        model,
        file,
        filename,
        api_key=None,
        api_base=None,
        api_type=None,
        api_version=None,
        organization=None,
        **params,
    ):
        requestor, files, data = cls._prepare_request(
            file=file,
            filename=filename,
            model=model,
            api_key=api_key,
            api_base=api_base,
            api_type=api_type,
            **params,
        )
        url = cls._get_url("transcriptions")
        response, _, api_key = await requestor.arequest(
            "post", url, files=files, params=data
        )
        return util.convert_to_openai_object(
            response, api_key, api_version, organization
        )

    @classmethod
    async def atranslate_raw(
        cls,
        model,
        file,
        filename,
        api_key=None,
        api_base=None,
        api_type=None,
        api_version=None,
        organization=None,
        **params,
    ):
        requestor, files, data = cls._prepare_request(
            file=file,
            filename=filename,
            model=model,
            api_key=api_key,
            api_base=api_base,
            api_type=api_type,
            **params,
        )
        url = cls._get_url("translations")
        response, _, api_key = await requestor.arequest(
            "post", url, files=files, params=data
        )
        return util.convert_to_openai_object(
            response, api_key, api_version, organization
        )



"""
This module helps make data libraries like `numpy` and `pandas` optional dependencies.

The libraries add up to 130MB+, which makes it challenging to deploy applications
using this library in environments with code size constraints, like AWS Lambda.

This module serves as an import proxy and provides a few utilities for dealing with the optionality.

Since the primary use case of this library (talking to the OpenAI API) doesn’t generally require data libraries,
it’s safe to make them optional. The rare case when data libraries are needed in the client is handled through
assertions with instructive error messages.

See also `setup.py`.

"""
try:
    import numpy
except ImportError:
    numpy = None

try:
    import pandas
except ImportError:
    pandas = None

HAS_NUMPY = bool(numpy)
HAS_PANDAS = bool(pandas)

INSTRUCTIONS = """

OpenAI error: 

    missing `{library}` 

This feature requires additional dependencies:

    $ pip install openai[datalib]

"""

NUMPY_INSTRUCTIONS = INSTRUCTIONS.format(library="numpy")
PANDAS_INSTRUCTIONS = INSTRUCTIONS.format(library="pandas")


class MissingDependencyError(Exception):
    pass


def assert_has_numpy():
    if not HAS_NUMPY:
        raise MissingDependencyError(NUMPY_INSTRUCTIONS)


def assert_has_pandas():
    if not HAS_PANDAS:
        raise MissingDependencyError(PANDAS_INSTRUCTIONS)



from openai import util
from openai.api_resources.abstract import (
    DeletableAPIResource,
    ListableAPIResource,
    CreateableAPIResource,
)
from openai.error import InvalidRequestError, APIError


class Deployment(CreateableAPIResource, ListableAPIResource, DeletableAPIResource):
    OBJECT_NAME = "deployments"

    @classmethod
    def _check_create(cls, *args, **kwargs):
        typed_api_type, _ = cls._get_api_type_and_version(
            kwargs.get("api_type", None), None
        )
        if typed_api_type not in (util.ApiType.AZURE, util.ApiType.AZURE_AD):
            raise APIError(
                "Deployment operations are only available for the Azure API type."
            )

        if kwargs.get("model", None) is None:
            raise InvalidRequestError(
                "Must provide a 'model' parameter to create a Deployment.",
                param="model",
            )

        scale_settings = kwargs.get("scale_settings", None)
        if scale_settings is None:
            raise InvalidRequestError(
                "Must provide a 'scale_settings' parameter to create a Deployment.",
                param="scale_settings",
            )

        if "scale_type" not in scale_settings or (
            scale_settings["scale_type"].lower() == "manual"
            and "capacity" not in scale_settings
        ):
            raise InvalidRequestError(
                "The 'scale_settings' parameter contains invalid or incomplete values.",
                param="scale_settings",
            )

    @classmethod
    def create(cls, *args, **kwargs):
        """
        Creates a new deployment for the provided prompt and parameters.
        """
        cls._check_create(*args, **kwargs)
        return super().create(*args, **kwargs)

    @classmethod
    def acreate(cls, *args, **kwargs):
        """
        Creates a new deployment for the provided prompt and parameters.
        """
        cls._check_create(*args, **kwargs)
        return super().acreate(*args, **kwargs)

    @classmethod
    def _check_list(cls, *args, **kwargs):
        typed_api_type, _ = cls._get_api_type_and_version(
            kwargs.get("api_type", None), None
        )
        if typed_api_type not in (util.ApiType.AZURE, util.ApiType.AZURE_AD):
            raise APIError(
                "Deployment operations are only available for the Azure API type."
            )

    @classmethod
    def list(cls, *args, **kwargs):
        cls._check_list(*args, **kwargs)
        return super().list(*args, **kwargs)

    @classmethod
    def alist(cls, *args, **kwargs):
        cls._check_list(*args, **kwargs)
        return super().alist(*args, **kwargs)

    @classmethod
    def _check_delete(cls, *args, **kwargs):
        typed_api_type, _ = cls._get_api_type_and_version(
            kwargs.get("api_type", None), None
        )
        if typed_api_type not in (util.ApiType.AZURE, util.ApiType.AZURE_AD):
            raise APIError(
                "Deployment operations are only available for the Azure API type."
            )

    @classmethod
    def delete(cls, *args, **kwargs):
        cls._check_delete(*args, **kwargs)
        return super().delete(*args, **kwargs)

    @classmethod
    def adelete(cls, *args, **kwargs):
        cls._check_delete(*args, **kwargs)
        return super().adelete(*args, **kwargs)

    @classmethod
    def _check_retrieve(cls, *args, **kwargs):
        typed_api_type, _ = cls._get_api_type_and_version(
            kwargs.get("api_type", None), None
        )
        if typed_api_type not in (util.ApiType.AZURE, util.ApiType.AZURE_AD):
            raise APIError(
                "Deployment operations are only available for the Azure API type."
            )

    @classmethod
    def retrieve(cls, *args, **kwargs):
        cls._check_retrieve(*args, **kwargs)
        return super().retrieve(*args, **kwargs)

    @classmethod
    def aretrieve(cls, *args, **kwargs):
        cls._check_retrieve(*args, **kwargs)
        return super().aretrieve(*args, **kwargs)
